=begin
# Author:         tomot
# Name:           DoorToolFrame
# Description:    Create a simple DoorFrame 
# Usage:          click 3 points, into an EXISTING OPENING, creates a DoorFrame into the opening
# Date:           2008 
# Type:           Tool
# Revised:        
#------------------------------------------------------------------------------------------------
=end
require 'sketchup.rb'
require 'doortools/3PointTool'

class DoorToolFrame < ThreePointTool

def initialize
  
    super # calls the initialize method in the ThreePointTool class
    # sets the default Door settings
    $axHB38 = 6.0. inch if not $axHB38      # door frame/wall thickness
    $qcMU62f = 1.5. inch if not $qcMU62f      # door frame thickness
                
    # Set the layer names
    $drDR00= "室内门窗" if not $drDR00        #layer for door
    @swinglayer= "门开启方向" if not @swinglayer  #layer for doorswing
         
    # Dialog box
    prompts = ["默认图层", "墙体厚度", "门框架厚度"]
    values = [$drDR00, $axHB38, $qcMU62f]
    results = inputbox prompts, values,  "门框架参数"
    
    return nil if not results
     $drDR00, $axHB38, $qcMU62f = results
       
end # initialize
#----------------------
def create_geometry
#----------------------
    
    model=Sketchup.active_model
    #------Set and activate the Doors layer
    layers = model.layers
    layers.add ($drDR00)
    activelayer = model.active_layer=layers[$drDR00]
    layer=model.active_layer
    
    model.start_operation "创建门框架"
    entities=model.active_entities
    
    # creates a new point from p1 in the direction of p2-p3 with length d
    # params are Point3d, 2 vertices, a length, returns a Point3d
    def translate(p1, p2, p3, d)
        v = p3 - p2
        v.length = d
        trans=Geom::Transformation.translation(v)
        return p1.transform(trans)
    end 
    
    #---Rotation definition for around z axis: 
    def translate_rotate_z(p1, p2, d, angle)
        v = p2 - p1
        v.length = d
        translate = Geom::Transformation.translation(v)
        p_temp = p1.transform(translate)
        rotate = Geom::Transformation.rotation( p1, Geom::Vector3d.new(0,0,1),angle)
        result = p_temp.transform(rotate)
        return result
    end
    
    pi = 3.141592653589793  # 180 degree angle = pi radians
    vecz = Geom::Vector3d.new(0, 0, 1)
    vecy = Geom::Vector3d.new(0, 1, 0)     
    vecx = Geom::Vector3d.new(1, 0, 0) 
    
    #------create a new set of points from the original 3 point pick
    @pt0=Geom::Point3d.new(@pts[0][0], @pts[0][1], @pts[0][2])
    @pt1=Geom::Point3d.new(@pts[1][0], @pts[1][1], @pts[1][2])
    @pt2=Geom::Point3d.new(@pts[2][0], @pts[2][1], @pts[2][2])
    @pt3=Geom::Point3d.new(@pts[3][0], @pts[3][1], @pts[3][2])
    
    @pt0j=translate(@pt0, @pt0, @pt3, $qcMU62f)
    @pt1j=translate(@pt1, @pt1, @pt2, $qcMU62f)
    @pt0jj=translate(@pt0j, @pt0j, @pt1j, $qcMU62f)
    @pt3jj=translate(@pt3, @pt3, @pt2, $qcMU62f)
    @pt1jj=translate(@pt1j, @pt1j, @pt0j, $qcMU62f)
    @pt2jj=translate(@pt2, @pt2, @pt3, $qcMU62f)
        
    #------Do Door Frame
    group=entities.add_group
    entities=group.entities
    base=entities.add_face(@pt3, @pt0, @pt1, @pt2, @pt2jj, @pt1jj, @pt0jj, @pt3jj)
    base.pushpull $axHB38   
    
    #------Set and activate the Door Swing layer
    layers = model.layers
    layers.add (@swinglayer)
    activelayer = model.active_layer=layers[@swinglayer]
    layer=model.active_layer
    
    #------Draw Door Frame Profile 
    $pt8=translate_rotate_z($pt3jj, $pt2jj, $axHB38, pi/2)
    $pt9=translate_rotate_z($pt2jj, $pt3jj, -$axHB38, pi/2)
    
    $pt17=translate($pt8, $pt8, $pt9, -$qcMU62f)
    $pt18=translate($pt9, $pt9, $pt8, -$qcMU62f)
    
    group=entities.add_group
    entities=group.entities
    model.entities.add_line($pt18, $pt9, $pt2jj, $pt2, $pt18)   
    model.entities.add_line($pt17, $pt3, $pt3jj, $pt8, $pt17) 
    
    #------Reset the layer back to default layer [0]
	layers = model.layers
    activelayer = model.active_layer=layers[0]
	layer=model.active_layer
    
    model.commit_operation
end
  
end # class DoorToolFrame
