=begin
# Author:         tomot
# Name:           DoorTool Glass
# Description:    Create a simple Door Glass 
# Usage:          click 3 points, into an EXISTING OPENING, creates glazing with offset
# Date:           2008 
# Type:           Tool
# Revised:        
#                 
#------------------------------------------------------------------------------------------------
=end
require 'sketchup.rb'
require 'doortools/3PointTool'

class DoorToolGlass < ThreePointTool

def initialize
    
    super # calls the initialize method in the ThreePointTool class
    # sets the default Door settings
    $dlCY19 = 0.0.inch if not $dlCY19     # offset vertical
    $slDW15 = 0.0.inch if not $slDW15     # offset horizontal
    $gkQN16 = 0.inch if not @pthick       # glass thickness
    $hnWD18 = 0.75.inch if not $hnWD18    # glass setback
    
    # Set the layer names
    $drDR00= "室内门" if not $drDR00        #layer for door
    @swinglayer= "门开启方向" if not @swinglayer  #layer for doorswing
    
    # Dialog box
    prompts = ["默认图层","玻璃垂直偏移", "玻璃水平偏移", "玻璃厚度 ", "玻璃进深尺寸 "]
    values = [$drDR00, $dlCY19, $slDW15, $gkQN16, $hnWD18]
    results = inputbox prompts, values, "门附件-玻璃参数"

    return if not results
    $drDR00, $dlCY19, $slDW15, $gkQN16, $hnWD18= results
    
end # initialize
#----------------------
def create_geometry
#----------------------

    model=Sketchup.active_model
    #------Set and activate the Doors layer
    layers = model.layers
    layers.add ($drDR00)
    activelayer = model.active_layer=layers[$drDR00]
    layer=model.active_layer

    model.start_operation "Create Glazing"
    entities=model.active_entities
    
    # creates a new point from p1 in the direction of p2-p3 with length d
    # params are Point3d, 2 vertices, a length, returns a Point3d
    def translate(p1, p2, p3, d)
        v = p3 - p2
        v.length = d
        trans=Geom::Transformation.translation(v)
        return p1.transform(trans)
    end   
         
    group=entities.add_group
    entities=group.entities
        
    #------create a new set of points from the original 3 point pick
    @pt0=Geom::Point3d.new(@pts[0][0], @pts[0][1], @pts[0][2])
    @pt1=Geom::Point3d.new(@pts[1][0], @pts[1][1], @pts[1][2])
    @pt2=Geom::Point3d.new(@pts[2][0], @pts[2][1], @pts[2][2])
    @pt3=Geom::Point3d.new(@pts[3][0], @pts[3][1], @pts[3][2])
    
    #------Deterine required points
    @pt0j=translate(@pt0, @pt0, @pt3, $slDW15)
    @pt1j=translate(@pt1, @pt1, @pt2, $slDW15 )
    @pt2j=translate(@pt2, @pt2, @pt1, $slDW15)
    @pt3j=translate(@pt3, @pt3, @pt0, $slDW15)
    @pt0jj=translate(@pt0j, @pt0j, @pt1j, $dlCY19)
    @pt3jj=translate(@pt3j, @pt3j, @pt2j, $dlCY19)
    @pt1jj=translate(@pt1j, @pt1j, @pt0j, $dlCY19)
    @pt2jj=translate(@pt2j, @pt2j, @pt3j, $dlCY19)
    
    #------Do the Glazing 
    group=entities.add_group
    entities=group.entities
    base=entities.add_face(@pt0jj, @pt1jj, @pt2jj, @pt3jj)
    base.material=Sketchup::Color.new(163,204,204) #use RGB Color numbers
    base.material.alpha = 0.6 
    base.pushpull -$gkQN16
    self.reset
    normal=base.normal
    normal.length= -$hnWD18-$gkQN16
    tr=Geom::Transformation.translation(normal.reverse)
    group.transform! tr
   
    #------Reset the layer back to default layer [0]
	layers = model.layers
    activelayer = model.active_layer=layers[0]
	layer=model.active_layer

    model.commit_operation
end

end  # class DoorToolGlass 
 

