=begin
# Author:         tomot
# Name:           DoorToolLeft
# Description:    Create a simple Door
# Usage:          click 3 points, into an EXISTING OPENING, creates a Left Hand or Left Hand Reverse Door
# Date:           2008 
# Type:           Tool
# Revised:        I kept the globals which are used in the dialog box. This way the user can retain the values he/she enter
#                 for that particular session. I use an alpha numeric random sequence, such as $tpAW87. This will avoid global 
#                 naming conflicts 99.9% of the time with other rubies. 
#------------------------------------------------------------------------------------------------
=end
require 'sketchup.rb'

class DoorToolLeft
def initialize
    @ip = Sketchup::InputPoint.new
    @ip1 = Sketchup::InputPoint.new
    reset

    # sets the default Door settings
    $axHB38 = 6.0. inch if not $axHB38              # door frame/wall thickness
    $plVC09s = 0.0. inch if not $plVC09s            # door frame thickness
    $qcMU62s = 1.5. inch if not $qcMU62s            # door thickness
    $sxTF94s = 6.0. inch if not $sxTF94s            # door style width
    $ocZK82s = 6.0. inch if not @dtthick            # door top rail width
    $mfQH29s = 12.0. inch if not $mfQH29s           # door btm rail width
    @ldlatchht = 40.0. inch if not @ldlatchht       # door latch height
    $poDL49 = "左手外开" if not $poDL49            # right hand or right hand reverse
    $blMD23 = "平板门" if not $blMD23                 # slab,panel,panel&glass,glass
    $dpSW39 = "否" if not $dpSW39                   # panic rail No or Yes
    $fkFS61 = "把手" if not $fkFS61                # hardware Lever, Knob, None   
    
    # Set the layer names
    $drDR00= "室内门" if not $drDR00           # layer for door
    @swinglayer= "门开启方向" if not @swinglayer     # layer for doorswing
    
   # Dialog box
    lswing = ["左手外开", "左手内开"]
    lstyle = ["平板门", "套框平板", "套框中空", "套框玻璃", "纯玻璃"]
    lpanic = ["是","否"]
    lhware = ["把手", "旋钮", "无"]
    enums = ["", "", lswing.join("|"), lstyle.join("|"), "", "", "", "", lpanic.join("|"), lhware.join("|")]
    prompts = ["默认图层", "墙厚", "门开启方向", "门样式",  "门厚", "门样式宽度 ", "门顶线宽度", "门底线宽度 ", "横档宽度", "门锁"]
    values = [$drDR00, $axHB38, $poDL49, $blMD23, $qcMU62s, $sxTF94s, $ocZK82s, $mfQH29s, $dpSW39, $fkFS61]
    results = inputbox prompts, values, enums, "左手开门参数"
    
    return nil if not results
    $drDR00, $axHB38, $poDL49, $blMD23, $qcMU62s, $sxTF94s, @dtthick, $mfQH29s, $dpSW39, $fkFS61 = results
    
    if ($sxTF94s < 3)
       UI.messagebox( "门样式宽度不低于3英寸 ")
    return nil
    end
    
    if ($sxTF94s > 12)
       UI.messagebox( "门样式宽度不高于12英寸 ")
    return nil
    end
    
    if ($mfQH29s < 3)
       UI.messagebox( "门底线宽度不低于3英寸")
    return nil
    end
    
    if ($mfQH29s > 12)
       UI.messagebox( "门底线宽度不高于12英寸")
    return nil
    end
    
    if ($ocZK82s < 3)
       UI.messagebox( "门顶线宽度不低于3英寸")
    return nil
    end
    
    if ($ocZK82s > 12)
       UI.messagebox( "门顶线宽度不高于12英寸")
    return nil
    end
  
end #end initialize
#---------------------------------------------------------------------------
def reset
    @pts = []
    @state = 0
    @ip1.clear
    @drawn = false
    Sketchup::set_status_text "", SB_VCB_LABEL
    Sketchup::set_status_text "", SB_VCB_VALUE
    Sketchup::set_status_text "点击左上角开始制作左手开门"
    @shift_down_time = Time.now
end

def activate
    self.reset
end

def deactivate(view)
    view.invalidate if @drawn
end

def set_current_point(x, y, view)
    if( !@ip.pick(view, x, y, @ip1) )
        return false
    end
    need_draw = true
    
    # Set the tooltip that will be displayed
    view.tooltip = @ip.tooltip
        
    # Compute points
    case @state
    when 0
        @pts[0] = @ip.position
        @pts[4] = @pts[0]
        need_draw = @ip.display? || @drawn
    when 1
        @pts[1] = @ip.position
        @length = @pts[0].distance @pts[1]
        Sketchup::set_status_text @length.to_s, SB_VCB_VALUE
    when 2
        pt1 = @ip.position
        pt2 = pt1.project_to_line @pts
        vec = pt1 - pt2

        @pt1 = @ip.position
        @pt2 = pt1.project_to_line @pts
        @vec = @pt1 - @pt2


        @width = vec.length
        if( @width > 0 )
            # test for a square
            square_point = pt2.offset(vec, @length)
            if( view.pick_helper.test_point(square_point, x, y) )
                @width = @length
                @pts[2] = @pts[1].offset(vec, @width)
                @pts[3] = @pts[0].offset(vec, @width)
                view.tooltip = "Square"
            else
                @pts[2] = @pts[1].offset(vec)
                @pts[3] = @pts[0].offset(vec)
            end
        else
            @pts[2] = @pts[1]
            @pts[3] = @pts[0]
        end
        Sketchup::set_status_text @width.to_s, SB_VCB_VALUE
    end

    view.invalidate if need_draw
end

def onMouseMove(flags, x, y, view)
    self.set_current_point(x, y, view)
end

#--------------------------------------------------------------------------
def create_rectangle
#--------------------------------------------------------------------------
      
    model=Sketchup.active_model
    #------Set and activate the Doors layer
    layers = model.layers
    layers.add ($drDR00)
    activelayer = model.active_layer=layers[$drDR00]
    layer=model.active_layer
    
    model.start_operation "创建门"
    entities=model.active_entities
    
    # creates a new point from p1 in the direction of p2-p3 with length d
    # params are Point3d, 2 vertices, a length, returns a Point3d
    def translate(p1, p2, p3, d)
        v = p3 - p2
        v.length = d
        trans=Geom::Transformation.translation(v)
        return p1.transform(trans)
    end  

    #---Rotation definition for around z axis: 
    def translate_rotate_z(p1, p2, d, angle)
        v = p2 - p1
        v.length = d
        translate = Geom::Transformation.translation(v)
        p_temp = p1.transform(translate)
        rotate = Geom::Transformation.rotation( p1, Geom::Vector3d.new(0,0,1),angle)
        result = p_temp.transform(rotate)
        return result
    end
    
    pi = 3.141592653589793  # 180 degree angle = pi radians
    vecz = Geom::Vector3d.new(0, 0, 1)
    vecy = Geom::Vector3d.new(0, 1, 0)     
    vecx = Geom::Vector3d.new(1, 0, 0) 
            
    #------create a new set of points from the original 3 point pick
    @pt0=Geom::Point3d.new(@pts[0][0], @pts[0][1], @pts[0][2])
    @pt1=Geom::Point3d.new(@pts[1][0], @pts[1][1], @pts[1][2])
    @pt2=Geom::Point3d.new(@pts[2][0], @pts[2][1], @pts[2][2])
    @pt3=Geom::Point3d.new(@pts[3][0], @pts[3][1], @pts[3][2])
    
    
    @pt0j=translate(@pt0, @pt0, @pt3, $plVC09s)
    @pt1j=translate(@pt1, @pt1, @pt2, $plVC09s)
    @pt0jj=translate(@pt0j, @pt0j, @pt1j, $plVC09s)
    @pt3jj=translate(@pt3, @pt3, @pt2, $plVC09s)
    @pt1jj=translate(@pt1j, @pt1j, @pt0j, $plVC09s)
    @pt2jj=translate(@pt2, @pt2, @pt3, $plVC09s)
    @pt0jja=translate(@pt0jj, @pt0jj, @pt1j, $sxTF94s) 
    @pt3jja=translate(@pt3jj, @pt3jj, @pt2, $sxTF94s) 
    @pt1jja=translate(@pt1jj, @pt1jj, @pt0j, $sxTF94s) 
    @pt2jja=translate(@pt2jj, @pt2jj, @pt3, $sxTF94s) 
    @pt3jjb=translate(@pt3jja, @pt3jja, @pt0jja, $mfQH29s) 
    @pt2jjb=translate(@pt2jja, @pt2jja, @pt1jja, $mfQH29s) 
    @pt0jjb=translate(@pt0jja, @pt0jja, @pt3jja, @dtthick) 
    @pt1jjb=translate(@pt1jja, @pt1jja, @pt2jja, @dtthick) 
    #-----width of door
    @dwidth=@pt0jj.distance @pt1jj
    #-----width of door/2
    @pt4=translate(@pt0, @pt0, @pt1, @dwidth/2)
    @pt5=translate(@pt3, @pt3, @pt2, @dwidth/2) 
    #-----Escution Plate
    @pt3jjh=translate(@pt3jj, @pt3jj, @pt0jj, @ldlatchht+$sxTF94s/2) 
    @pt3jjc=translate(@pt3jja, @pt3jja, @pt0jja, @ldlatchht+$sxTF94s/2) 
    @pt3jjhl=translate(@pt3jj, @pt3jj, @pt0jj, @ldlatchht-$sxTF94s/2) 
    @pt3jjcl=translate(@pt3jja, @pt3jja, @pt0jja, @ldlatchht-$sxTF94s/2) 
    #-----Do Door Knob 
    @pt3jjhc=translate(@pt3jjh, @pt3jjh, @pt3jjc, $sxTF94s/2)
    @pt3jjhcl=translate(@pt3jjhl, @pt3jjhl, @pt3jjcl, $sxTF94s/2)
    @ptknob=translate(@pt3jjhc, @pt3jjhc, @pt3jjhcl, @ldlatchht-37)
    #-----Panic Rail   
    @pt2jjc=translate(@pt2jja, @pt2jja, @pt1jja, @ldlatchht+$sxTF94s/2)
    @pt2jjcl=translate(@pt2jja, @pt2jja, @pt1jja, @ldlatchht-$sxTF94s/2)
    #-----Door Handle 
    @handle = 8.0. inch if not @handle
    @handth = 0.375. inch if not @handth
    @pth=translate(@pt3jjh, @pt3jjh, @pt3jjhc, $qcMU62s/4)
    @pthl=translate(@pt3jjhl, @pt3jjhl, @pt3jjhcl, $qcMU62s/4)
    @ptknob2=translate(@pth, @pth, @pthl, @ldlatchht-37)
    @pthh=translate(@ptknob2, @ptknob2, @pth, -0.65) 
    @pthhl=translate(@ptknob2, @ptknob2, @pthl, -0.65)  
    @pthc=translate(@ptknob, @ptknob, @pt3jjhc, -0.65)
    @pthcl=translate(@ptknob, @ptknob, @pt3jjhcl, -0.65)
    @pth2g=translate(@pthh, @pthh, @pthc, @handle)
    @pth2gl=translate(@pthhl, @pthhl, @pthcl, @handle)
    
#---------------------------------------------------------------
# Do Left Hand Door
#---------------------------------------------------------------
if( $poDL49 == "左手外开") 

    if( $fkFS61 == "把手" ) 
        #-----Do Escution Plate
        group=entities.add_group
        entities=group.entities
        base=entities.add_face(@pt3jjcl, @pt3jjc, @pt3jjh, @pt3jjhl)
        base.pushpull -($qcMU62s+ $qcMU62s/8)
        #-----Do Escution Plate other side
        base=entities.add_face(@pt3jjcl, @pt3jjc, @pt3jjh, @pt3jjhl)
        base.pushpull $qcMU62s/8
        #-----Do Door Knob 
        @ptknob1=translate(@pt3jjhc, @pt3jjhc, @pt3jjhcl, @ldlatchht-37)
        @Vec0 = @pt1 - @pt0
        @Vec1 = @pt0 - @pt1
        @Vec2 = @pt0 - @pt3
        # The cross product of two vectors produces a third vector which 
        #is perpendicular to the plane in which the first two lie.
        @Vec3 = @Vec1 * @Vec2
        group=entities.add_group
        entities=group.entities
        circle = entities.add_circle(@ptknob1, @Vec3, 3/2, 24)
        hware = entities.add_face(circle)
        hware.pushpull $qcMU62s*2
        hware.pushpull $qcMU62s
        #-----Do Door Handle 
        group=entities.add_group
        handle = group.entities.add_face(@pthc, @pth2g, @pth2gl, @pthcl) 
        handle.pushpull -@handth     
        #-----Transposed location
        normal = handle.normal
        normal.length = @handth*3+@handth/2 
        tr = Geom::Transformation.translation(normal)
        group.transform! tr
        #-----Do Door Handle other side 
        group=entities.add_group
        handle1 = group.entities.add_face(@pthc, @pth2g, @pth2gl, @pthcl) 
        handle1.pushpull @handth   
        #-----Transposed location
        normal = handle1.normal
        normal.length = @handth*7+@handth/2 
        tr = Geom::Transformation.translation(normal)
        group.transform! tr
    end #if
    
    if( $fkFS61 == "旋钮" ) 
        #-----Do Escution Plate
        group=entities.add_group
        entities=group.entities
        base=entities.add_face(@pt3jjcl, @pt3jjc, @pt3jjh, @pt3jjhl)
        base.pushpull -($qcMU62s+ $qcMU62s/8)
        #-----Do Escution Plate other side
        base=entities.add_face(@pt3jjcl, @pt3jjc, @pt3jjh, @pt3jjhl)
        base.pushpull $qcMU62s/8
        #-----Do Door Knob 
        @ptknob1=translate(@pt3jjhc, @pt3jjhc, @pt3jjhcl, @ldlatchht-37)
        @Vec0 = @pt1 - @pt0
        @Vec1 = @pt0 - @pt1
        @Vec2 = @pt0 - @pt3
        # The cross product of two vectors produces a third vector which 
        #is perpendicular to the plane in which the first two lie.
        @Vec3 = @Vec1 * @Vec2
        group=entities.add_group
        entities=group.entities
        circle = entities.add_circle(@ptknob1, @Vec3, 3/2, 24)
        hware = entities.add_face(circle)
        hware.pushpull $qcMU62s*2
        hware.pushpull $qcMU62s
    end #if
    
    if( $dpSW39 != "否" ) 
        #------Panic Rail 
        group=entities.add_group
        rail=group.entities.add_face(@pt3jjc, @pt2jjc, @pt2jjcl, @pt3jjcl)
        rail.pushpull $qcMU62s
    end #if  
    
    if( $blMD23 == "平板门" )     
        #------Do Slab Door 
        group=entities.add_group
        entities=group.entities
        slab=entities.add_face(@pt0jj, @pt1jj, @pt2jj, @pt3jj)
        slab.pushpull $qcMU62s
    end  #if 
 
    if( $blMD23 == "套框平板" )      
        #------Do Slab Door 
        group=entities.add_group
        entities=group.entities
        panel=entities.add_face(@pt0jj, @pt1jj, @pt2jj, @pt3jj)
        panel.pushpull $qcMU62s
        #------Cut Panel Opening
        panel=entities.add_face(@pt0jjb, @pt1jjb, @pt2jjb, @pt3jjb)
        panel.pushpull -$qcMU62s
        #------Install Panel
        group=entities.add_group
        entities=group.entities
        panel=entities.add_face(@pt0jjb, @pt1jjb, @pt2jjb, @pt3jjb)
        panel.pushpull $qcMU62s/3
    
        normal = base.normal
        normal.length = -$qcMU62s*2/3
        tr = Geom::Transformation.translation(normal)
        group.transform! tr
    end #if    
 
    if( $blMD23 == "套框中空" )      
        #------Do Slab Door 
        group=entities.add_group
        entities=group.entities
        panel=entities.add_face(@pt0jj, @pt1jj, @pt2jj, @pt3jj)
        panel.pushpull $qcMU62s
        #------Cut Panel Opening
        panel=entities.add_face(@pt0jjb, @pt1jjb, @pt2jjb, @pt3jjb)
        panel.pushpull -$qcMU62s
    end #if  

    if( $blMD23 == "套框玻璃" ) 
        #------Do Slab Door 
        group=entities.add_group
        entities=group.entities
        panel=entities.add_face(@pt0jj, @pt1jj, @pt2jj, @pt3jj)
        panel.pushpull  $qcMU62s
        #------Cut Panel Opening
        panel=entities.add_face(@pt0jjb, @pt1jjb, @pt2jjb, @pt3jjb)
        panel.pushpull -$qcMU62s
        #------Install Glazing Panel
        group=entities.add_group
        entities=group.entities
        glaz=entities.add_face(@pt2jjb, @pt3jjb, @pt0jjb, @pt1jjb)
        glaz.material=Sketchup::Color.new(163,204,204) #use RGB Color numbers
        glaz.material.alpha = 0.6 

        normal = glaz.normal
        normal.length = -$qcMU62s/2
        tr = Geom::Transformation.translation(normal.reverse)
        group.transform! tr
    end #if   

    if( $blMD23 == "玻璃" )     
        #------Do Slab Door 
        group=entities.add_group
        entities=group.entities
        glass=entities.add_face(@pt0jj, @pt1jj, @pt2jj, @pt3jj)
        glass.material=Sketchup::Color.new(163,204,204) #use RGB Color numbers
        glass.material.alpha = 0.6 
        glass.pushpull $qcMU62s/3
        
        normal = glass.normal
        normal.length = -$qcMU62s/4
        tr = Geom::Transformation.translation(normal)
        group.transform! tr
    end #if  

    #------Set and activate the Door Swing layer
    layers = model.layers
    layers.add (@swinglayer)
    activelayer = model.active_layer=layers[@swinglayer]
    layer=model.active_layer
 
    #------Draw Left Hand Door arc  
    @pt10=translate_rotate_z(@pt2jj, @pt3jj, @dwidth, pi/2)
    
    @pt12b=translate(@pt2jj, @pt2jj, @pt3jj, @dwidth*0.5)
    @pt12c=translate_rotate_z(@pt12b, @pt2jj, -@dwidth*0.866, pi/2)
        
    @pt13b=translate(@pt2jj, @pt2jj, @pt3jj, @dwidth*0.2588)
    @pt13c=translate_rotate_z(@pt13b, @pt2jj, -@dwidth*0.9659, pi/2)
        
    @pt14b=translate(@pt2jj, @pt2jj, @pt3jj, @dwidth*0.7071)
    @pt14c=translate_rotate_z(@pt14b, @pt2jj, -@dwidth*0.7071, pi/2)
        
    @pt15b=translate(@pt2jj, @pt2jj, @pt3jj, @dwidth*0.8660)
    @pt15c=translate_rotate_z(@pt15b, @pt2jj, -@dwidth*0.5, pi/2)
        
    @pt16b=translate(@pt2jj, @pt2jj, @pt3jj, @dwidth*0.9659)
    @pt16c=translate_rotate_z(@pt16b, @pt2jj, -@dwidth*0.2588, pi/2)
        
    model.entities.add_line(@pt10, @pt13c, @pt12c, @pt14c, @pt15c, @pt16c, @pt3jj)  # line arc
    model.entities.add_line(@pt2jj, @pt10) #90 degree line
    
    #model.entities.add_arc(@pt2jj, vecx, vecz, @dwidth, -pi/2, -pi, 24)
   
end #end if Left Hand Door 
#--------------------------------------------------------------- 
# Do Left Hand Reverse Door
#---------------------------------------------------------------
if( $poDL49 == "左手内开" ) 

    if( $dpSW39 != "否" ) 
        #------Panic Rail 
        group=entities.add_group
        rail=group.entities.add_face(@pt3jjc, @pt2jjc, @pt2jjcl, @pt3jjcl)
        rail.pushpull -$qcMU62s
       
        normal = rail.normal
        normal.length = $axHB38
        tr = Geom::Transformation.translation(normal)
        group.transform! tr
    end #if     
      
    if( $blMD23 == "平板门" )     
        #------Do Slab Door 
        group=entities.add_group
        slab=group.entities.add_face(@pt0jj, @pt1jj, @pt2jj, @pt3jj)
        slab.pushpull -$qcMU62s
        
        normal = slab.normal
        normal.length = -$axHB38 
        tr = Geom::Transformation.translation(normal.reverse)
        group.transform! tr
    end  #if  

if( $blMD23 == "套框平板" )      
        #------Do Slab Door 
        group=entities.add_group
        panel=group.entities.add_face(@pt0jj, @pt1jj, @pt2jj, @pt3jj)
        panel.pushpull -$qcMU62s
        
        normal = panel.normal
        normal.length = $axHB38 
        tr = Geom::Transformation.translation(normal)
        group.transform! tr
        
        #------Cut Panel Opening
        panel=group.entities.add_face(@pt0jjb, @pt1jjb, @pt2jjb, @pt3jjb)
        panel.pushpull -$qcMU62s
        #------Install Panel
        group=entities.add_group
        panel=group.entities.add_face(@pt0jjb, @pt1jjb, @pt2jjb, @pt3jjb)
        panel.pushpull $qcMU62s/3
    
        normal = panel.normal
        normal.length = -$axHB38+$qcMU62s*2/3  #check setback
        tr = Geom::Transformation.translation(normal)
        group.transform! tr
    end #if  

    if( $blMD23 == "套框中空" )      
        #------Do Slab Door 
        group=entities.add_group
        open=group.entities.add_face(@pt0jj, @pt1jj, @pt2jj, @pt3jj)
        open.pushpull -$qcMU62s
        
        normal = open.normal
        normal.length = $axHB38 
        tr = Geom::Transformation.translation(normal)
        group.transform! tr
        
        #------Cut Panel Opening
        open=group.entities.add_face(@pt0jjb, @pt1jjb, @pt2jjb, @pt3jjb)
        open.pushpull -$qcMU62s
    end #if  

    if( $blMD23 == "套框玻璃" ) 
        #------Do Slab Door 
        group=entities.add_group
        pangl=group.entities.add_face(@pt0jj, @pt1jj, @pt2jj, @pt3jj)
        pangl.pushpull -$qcMU62s
        
        normal = pangl.normal
        normal.length = $axHB38 
        tr = Geom::Transformation.translation(normal)
        group.transform! tr
                
        #------Cut Panel Opening
        pangl=group.entities.add_face(@pt0jjb, @pt1jjb, @pt2jjb, @pt3jjb)
        pangl.pushpull -$qcMU62s
        
        #------Install Glazing Panel
        group=entities.add_group
        glaz=group.entities.add_face(@pt2jjb, @pt3jjb, @pt0jjb, @pt1jjb)
        glaz.material=Sketchup::Color.new(163,204,204) #use RGB Color numbers
        glaz.material.alpha = 0.6 

        normal = glaz.normal
        normal.length = -$axHB38+$qcMU62s/2
        tr = Geom::Transformation.translation(normal.reverse)
        group.transform! tr
    end #if   
 
    if( $blMD23 == "玻璃" )     
        #------Do Slab Door 
        group=entities.add_group
        glass=group.entities.add_face(@pt0jj, @pt1jj, @pt2jj, @pt3jj)
        glass.material=Sketchup::Color.new(163,204,204) #use RGB Color numbers
        glass.material.alpha = 0.6 
        glass.pushpull -$qcMU62s/3
        
        normal = glass.normal
        normal.length = $axHB38-$qcMU62s/4
        tr = Geom::Transformation.translation(normal)
        group.transform! tr
    end #if  
    
    if( $fkFS61 == "把手" )    
        #-----Do Escution Plate
        group=entities.add_group
        entities=group.entities
        base=entities.add_face(@pt3jjcl, @pt3jjc, @pt3jjh, @pt3jjhl)
        base.pushpull $qcMU62s+ $qcMU62s/8
        #-----Do Escution Plate other side
        base=entities.add_face(@pt3jjcl, @pt3jjc, @pt3jjh, @pt3jjhl)
        base.pushpull $qcMU62s/8
    
        normal = base.normal
        normal.length = -$axHB38
        tr = Geom::Transformation.translation(normal.reverse)
        group.transform! tr
    
        #-----Do Door Knob 
        @ptknob1=translate(@pt3jjhc, @pt3jjhc, @pt3jjhcl, @ldlatchht-37)
        @Vec0 = @pt1 - @pt0
        @Vec1 = @pt0 - @pt1
        @Vec2 = @pt0 - @pt3
        # The cross product of two vectors produces a third vector which 
        #is perpendicular to the plane in which the first two lie.
        @Vec3 = @Vec1 * @Vec2
        group=entities.add_group
        entities=group.entities
        circle = entities.add_circle(@ptknob1, @Vec3, 3/2, 24)
        hware = entities.add_face(circle)
        hware.pushpull -$qcMU62s*2
        hware.pushpull $qcMU62s
        #-----Do Door Handle 
        group=entities.add_group
        handle = group.entities.add_face(@pthc, @pth2g, @pth2gl, @pthcl) 
        handle.pushpull -@handth     
        #-----Transposed location
        normal = handle.normal
        normal.length = -@handth*3+@handth/2 
        tr = Geom::Transformation.translation(normal)
        group.transform! tr
        #-----Do Door Handle other side 
        group=entities.add_group
        handle1 = group.entities.add_face(@pthc, @pth2g, @pth2gl, @pthcl) 
        handle1.pushpull @handth   
        #-----Transposed location
        normal = handle1.normal
        normal.length = -@handth*7+@handth/2 
        tr = Geom::Transformation.translation(normal)
        group.transform! tr
    end #if  

    if( $fkFS61 == "旋钮" )    
        #-----Do Escution Plate
        group=entities.add_group
        entities=group.entities
        base=entities.add_face(@pt3jjcl, @pt3jjc, @pt3jjh, @pt3jjhl)
        base.pushpull $qcMU62s+ $qcMU62s/8
        #-----Do Escution Plate other side
        base=entities.add_face(@pt3jjcl, @pt3jjc, @pt3jjh, @pt3jjhl)
        base.pushpull $qcMU62s/8
    
        normal = base.normal
        normal.length = -$axHB38
        tr = Geom::Transformation.translation(normal.reverse)
        group.transform! tr
    
        #-----Do Door Knob 
        @ptknob1=translate(@pt3jjhc, @pt3jjhc, @pt3jjhcl, @ldlatchht-37)
        @Vec0 = @pt1 - @pt0
        @Vec1 = @pt0 - @pt1
        @Vec2 = @pt0 - @pt3
        # The cross product of two vectors produces a third vector which 
        #is perpendicular to the plane in which the first two lie.
        @Vec3 = @Vec1 * @Vec2
        group=entities.add_group
        entities=group.entities
        circle = entities.add_circle(@ptknob1, @Vec3, 3/2, 24)
        hware = entities.add_face(circle)
        hware.pushpull -$qcMU62s*2
        hware.pushpull $qcMU62s
    end #if
    
    #------Set and activate the Door Swing layer
    layers = model.layers
    layers.add (@swinglayer)
    activelayer = model.active_layer=layers[@swinglayer]
    layer=model.active_layer

    #------Draw Left Hand Reverse Door arc
    @pt8=translate_rotate_z(@pt3jj, @pt2jj, $axHB38, pi/2)
    @pt9=translate_rotate_z(@pt2jj, @pt3jj, -$axHB38, pi/2)
    @pt10=translate_rotate_z(@pt2jj, @pt3jj, -$axHB38-@dwidth, pi/2)
    
    @pt12b=translate(@pt9, @pt9, @pt8, @dwidth*0.5)
    @pt12c=translate_rotate_z(@pt12b, @pt9, @dwidth*0.866, pi/2)
        
    @pt13b=translate(@pt9, @pt9, @pt8, @dwidth*0.2588)
    @pt13c=translate_rotate_z(@pt13b, @pt9, @dwidth*0.9659, pi/2)
        
    @pt14b=translate(@pt9, @pt9, @pt8, @dwidth*0.7071)
    @pt14c=translate_rotate_z(@pt14b, @pt9, @dwidth*0.7071, pi/2)
        
    @pt15b=translate(@pt9, @pt9, @pt8, @dwidth*0.8660)
    @pt15c=translate_rotate_z(@pt15b, @pt9, @dwidth*0.5, pi/2)
        
    @pt16b=translate(@pt9, @pt9, @pt8, @dwidth*0.9659)
    @pt16c=translate_rotate_z(@pt16b, @pt9, @dwidth*0.2588, pi/2)
        
    model.entities.add_line(@pt10, @pt13c, @pt12c, @pt14c, @pt15c, @pt16c, @pt8)  # line arc
    model.entities.add_line(@pt9, @pt10) #90 degree line
    
    #model.entities.add_arc(@pt9, vecx, vecz, @dwidth,  0, pi/2, 24)

end #end if Left Hand Reverse Door
#------------------------------------------------------------------
    
    #------Reset layer back to layer [0]
	layers = model.layers
	activelayer = model.active_layer=layers[0]
	layer=model.active_layer
    
     model.commit_operation
end
#----------------------------------------------------------------------
def increment_state
    @state += 1
    case @state
    when 1
        @ip1.copy! @ip
        Sketchup::set_status_text "点击顶部对应的另一点作为第二点"
        Sketchup::set_status_text "", SB_VCB_LABEL
        Sketchup::set_status_text "", SB_VCB_VALUE
    when 2
        @ip1.clear
        Sketchup::set_status_text "点击底部作为第三点"
        Sketchup::set_status_text "", SB_VCB_LABEL
        Sketchup::set_status_text "", SB_VCB_VALUE
    when 3
        self.create_rectangle
    end
end

def onLButtonDown(flags, x, y, view)
    self.set_current_point(x, y, view)
    self.increment_state
    view.lock_inference
end

def onCancel(flag, view)
    view.invalidate if @drawn
    self.reset
end

# This is called when the user types a value into the VCB
def onUserText(text, view)
    # The user may type in something that we can't parse as a length
    # so we set up some exception handling to trap that
    begin
        value = text.to_l
    rescue
        # Error parsing the text
        UI.beep
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value
    
    case @state
    when 1
        # update the width
        vec = @pts[1] - @pts[0]
        if( vec.length > 0.0 )
            vec.length = value
            @pts[1] = @pts[0].offset(vec)
            view.invalidate
            self.increment_state
        end
    when 2
        # update the height
        vec = @pts[3] - @pts[0]
        if( vec.length > 0.0 )
            vec.length = value
            @pts[2] = @pts[1].offset(vec)
            @pts[3] = @pts[0].offset(vec)
            self.increment_state
        end
    end
end

def getExtents
    bb = Geom::BoundingBox.new
    case @state
    when 0
        # We are getting the first point
        if( @ip.valid? && @ip.display? )
            bb.add @ip.position
        end
    when 1
        bb.add @pts[0]
        bb.add @pts[1]
    when 2
        bb.add @pts
    end
    bb
end

def draw(view)
    @drawn = false
    
    # Show the current input point
    if( @ip.valid? && @ip.display? )
        @ip.draw(view)
        @drawn = true
    end

    # show the rectangle
    if( @state == 1 )
        # just draw a line from the start to the end point
        view.set_color_from_line(@ip1, @ip)
        inference_locked = view.inference_locked?
        view.line_width = 3 if inference_locked
        view.draw(GL_LINE_STRIP, @pts[0], @pts[1])
        view.line_width = 1 if inference_locked
        @drawn = true
    elsif( @state > 1 )
        # draw the curve
        view.drawing_color = "black"
        view.draw(GL_LINE_STRIP, @pts)
        @drawn = true
    end
end

def onKeyDown(key, rpt, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY && rpt == 1 )
        @shift_down_time = Time.now
        
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
            view.lock_inference
        elsif( @state == 0 )
            view.lock_inference @ip
        elsif( @state == 1 )
            view.lock_inference @ip, @ip1
        end
    end
end

def onKeyUp(key, rpt, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end

  
end # of class DoorToolLeft
