=begin
# Author:         tomot
# Name:           DoorToolLouver
# Description:    Create Louver 
# Usage:          click 3 points, into an EXISTING OPENING, creates a set of Louvers
#                 with options of setback from face and sides of opening.
# Date:           2008 
# Type:           Tool
# Revised:        
#------------------------------------------------------------------------------------------------------
=end
require 'sketchup.rb'
require 'doortools/3PointTool'

class DoorToolLouver < ThreePointTool

def initialize
    
    super # calls the initialize method in the ThreePointTool class
    # sets the default Window settings
    $adNI14 = 1.inch if not $adNI14       # Louver Spacing
    $klGY59 = 45 if not  $klGY59          # Louver angle
    $fgHJ12 = 1.5.inch if not $fgHJ12     # Louver width
    $klGY24 = 0.25.inch if not $klGY24    # Louver thickness
    $plBH85 = 0.inch if not $plBH85       # Louver offset from opening
    $buTB05 = 0.inch if not $buTB05       # Louver setback from face
    $etKU36= "Solid" if not $etKU36       # Solid or Glass 
    
    $fgHJ12 = 0.25.inch if $fgHJ12 <= 0.inch
    
    # Set the layer names
    $drDR00= "Doors" if not $drDR00        #layer for door
    @swinglayer= "DoorSwing" if not @swinglayer  #layer for doorswing
    
    # Dialog box
    ltype = ["实体", "玻璃"]
    enums = ["",ltype.join("|")]
    prompts = ["默认图层", "百叶类型", "百叶间距", "百叶角度", "百页宽度", "百叶厚度", "百叶偏移：从边 ","百叶进深位置：从面"]
    values = [ $drDR00, $etKU36, $adNI14, $klGY59, $fgHJ12, $klGY24, $plBH85, $buTB05]
    results = inputbox prompts, values, enums, "门附件-百叶参数"

    return if not results
    $drDR00, $ltype, $adNI14, $klGY59, $fgHJ12, $klGY24, $plBH85, $buTB05 = results
  
end # initialize
#----------------------
def create_geometry
#----------------------

    model=Sketchup.active_model
    #------Set and activate the Doors layer
    layers = model.layers
    layers.add ($drDR00)
    activelayer = model.active_layer=layers[$drDR00]
    layer=model.active_layer

    model.start_operation "创建百叶"
    entities=model.active_entities

    # creates a new point from p1 in the direction of p2-p3 with length d
    # params are Point3d, 2 vertices, a length, returns a Point3d
    def translate(p1, p2, p3, d)
      v = p3 - p2
      v.length = d
      trans=Geom::Transformation.translation(v)
      return p1.transform(trans)
    end   
    
    #---Rotation definition for around z axis: 
    def translate_rotate_z(p1, p2, d, angle)
        v = p2 - p1
        v.length = d
        translate = Geom::Transformation.translation(v)
        p_temp = p1.transform(translate)
        rotate = Geom::Transformation.rotation( p1, Geom::Vector3d.new(0,0,1),angle)
        result = p_temp.transform(rotate)
        return result
    end
    
    pi = 3.141592653589793  # 180 degree angle = pi radians
    vecz = Geom::Vector3d.new(0, 0, 1)
    vecy = Geom::Vector3d.new(0, 1, 0)     
    vecx = Geom::Vector3d.new(1, 0, 0) 

    #------create a new set of points from the original 3 point pick
    @pt0=Geom::Point3d.new(@pts[0][0],@pts[0][1],@pts[0][2])
    @pt1=Geom::Point3d.new(@pts[1][0],@pts[1][1],@pts[1][2])
    @pt2=Geom::Point3d.new(@pts[2][0],@pts[2][1],@pts[2][2])
    @pt3=Geom::Point3d.new(@pts[3][0],@pts[3][1],@pts[3][2])

    @pt0j=translate(@pt0, @pt0, @pt3, $plBH85)
    @pt1j=translate(@pt1, @pt1, @pt2, $plBH85)
    @pt2j=translate(@pt2, @pt2, @pt1, $plBH85)
    @pt3j=translate(@pt3, @pt3, @pt0, $plBH85)
   
    @pt0jj=translate(@pt0, @pt0, @pt1, $plBH85)
    @pt3jj=translate(@pt3j, @pt3j, @pt2j, $plBH85)
    @pt1jj=translate(@pt1, @pt1, @pt0, $plBH85)
    @pt2jj=translate(@pt2j, @pt2j, @pt3j, $plBH85)
    
    @xcomp= Math::cos($klGY59.degrees)*$fgHJ12
    @ycomp= Math::sin($klGY59.degrees)*$fgHJ12

    @pt4=translate(@pt1jj, @pt1jj, @pt2jj, @ycomp)
    @pt5=translate(@pt0jj, @pt0jj, @pt3jj, @ycomp)
    
    @pt6=translate_rotate_z(@pt5, @pt4, @xcomp, pi/2)
    @pt7=translate_rotate_z(@pt4, @pt5, -@xcomp, pi/2)
    
    @pt8=translate_rotate_z(@pt0jj, @pt1jj, @xcomp, pi/2)
    @pt9=translate_rotate_z(@pt1jj, @pt0jj, -@xcomp, pi/2)
    
    #------Offset from Face
    @pt8a=translate_rotate_z(@pt9, @pt8, -$buTB05, pi/2)
    @pt9a=translate_rotate_z(@pt8, @pt9, $buTB05, pi/2)
    
    @pt4a=translate_rotate_z(@pt5, @pt4, $buTB05, pi/2)
    @pt5a=translate_rotate_z(@pt4, @pt5, -$buTB05, pi/2)
#---------------------------------------------------------
if( $etKU36== "实体" ) 
#---------------------------------------------------------
    #------Width of opening
    @lOwidth=@pt0.distance @pt1
    #------Height of opening
    @lOheight=@pt0.distance @pt3
    
    #------Get the Num of Louvers req'd @lOheight/$adNI14
    @num_Louver=((@lOheight/$adNI14)+1).to_i 
    
    definitions=model.definitions
    count=definitions.add
    entities=count.entities
        
    #------Do 1st Sloped Louver
    base=entities.add_face(@pt8a,@pt9a,@pt4a,@pt5a)
    base.pushpull -$klGY24
    
    #------Do transform of location  
    t=Geom::Transformation.translation(Geom::Vector3d.new(0, 0, 0))
    entities=model.active_entities
    entities.add_instance(count, t)
    
    #------Copy Louvers to new locations
    i = 1
    while i < @num_Louver
       i = i + 1
       vec = @pt3 - @pt0              #example: vec = Vector3d(0, 0, -84)
       vec.length = $adNI14*(i-2)    #example: vec.length = 84.0
       t=Geom::Transformation.translation(Geom::Vector3d.new(0,$adNI14*(i-2), 0))
       t=Geom::Transformation.translation(vec)
       entities.add_instance(count, t)
    end
end #if     
#---------------------------------------------------------
if( $etKU36== "玻璃" ) 
#---------------------------------------------------------  
    #------Width of opening
    @lOwidth=@pt0.distance @pt1
    #------Height of opening
    @lOheight=@pt0.distance @pt3
    
    #------Get the Num of Louvers req'd @lOheight/$adNI14
    @num_Louver=((@lOheight/$adNI14)+1).to_i 
    
    definitions=model.definitions
    count=definitions.add
    entities=count.entities
        
    #------Do 1st Sloped Louver
    glass=entities.add_face(@pt5a,@pt4a,@pt9a,@pt8a)
    glass.material=Sketchup::Color.new(163,204,204) #use RGB Color numbers
    glass.material.alpha = 0.6 
    glass.pushpull -$klGY24
    
    #------Do transform of location  
    t=Geom::Transformation.translation(Geom::Vector3d.new(0, 0, 0))
    entities=model.active_entities
    entities.add_instance(count, t)
    
    #------Copy Louvers to new locations
    i = 1
    while i < @num_Louver
       i = i + 1
       vec = @pt3 - @pt0              #example: vec = Vector3d(0, 0, -84)
       vec.length = $adNI14*(i-2)    #example: vec.length = 84.0
       t=Geom::Transformation.translation(Geom::Vector3d.new(0,$adNI14*(i-2), 0))
       t=Geom::Transformation.translation(vec)
       entities.add_instance(count, t)
    end
end #if  

    #------Reset the layer back to default layer [0]
	layers = model.layers
    activelayer = model.active_layer=layers[0]
	layer=model.active_layer

    model.commit_operation
end

end # class DoorToolLouver
