=begin
# Author:         tomot
# Name:           DoorToolMullion 
# Description:    Create Mullions and infill panels for doors
# Usage:          click 3 points, on an EXISTING OPENING, creates mullions into the opening
# Date:           2008 
# Type:           Tool
# Revised:       
#
#------------------------------------------------------------------------------------------------------
=end
require 'sketchup.rb'
require 'doortools/3PointTool'

class DoorToolMullion < ThreePointTool

def initialize
    
    super # calls the initialize method in the ThreePointTool class
    # set the default Door Mullion settings
    $dpFT29 = 0.inch if not $dpFT29        # offset from opening
    $jwEN56 = 5 if not $jwEN56             # number rows of panels
    $wbTH20 = 3 if not $wbTH20             # number colums of panels
    $voVP26 = 1. inch if not $voVP26       # Mullion width
    $coKX56 = 1.5. inch if not $coKX56     # Mullion thickness
    $smBO73 = 2.0. inch if not $smBO73     # Total Panel 
    @gtype = "竖窗" if not @gtype      # Mullions, Panels, Mullions & Panels


    $jwEN56 = 1 if $jwEN56 < 1
    $wbTH20 = 1 if $wbTH20 < 1
    $voVP26 = 0.25.inch if $voVP26 <= 0.inch
    $coKX56 = 0.25.inch if $coKX56 <= 0.inch
    
    # Set the layer names
    $drDR00= "室内门" if not $drDR00        #layer for door
    @swinglayer= "门开启方向" if not @swinglayer  #layer for doorswing
     
    # Dialog box
    mtype = ["竖窗", "套框板", "竖窗与套框板"]
    enums = ["", mtype.join("|")]
    prompts = ["默认图层", "竖窗类型","竖窗偏移：沿中空部分 ", "套框竖棂", "套框横棂", "竖窗宽度", "竖窗厚度", "竖窗厚度+套框 "]
    values = [ $drDR00, @gtype, $dpFT29, $jwEN56, $wbTH20, $voVP26, $coKX56, $smBO73]
    results = inputbox prompts, values, enums, "门附件：竖窗参数"

    return if not results
    $drDR00, @gtype, $dpFT29, $jwEN56, $wbTH20, $voVP26, $coKX56,$smBO73 = results
    
end # initialize
#----------------------
def create_geometry
#----------------------

    model=Sketchup.active_model
    #------Set and activate the Doors layer
    layers = model.layers
    layers.add ($drDR00)
    activelayer = model.active_layer=layers[$drDR00]
    layer=model.active_layer
    
    model.start_operation "创建竖窗"
    entities=model.active_entities

    # creates a new point from p1 in the direction of p2-p3 with length d
    # params are Point3d, 2 vertices, a length, returns a Point3d
    def translate(p1, p2, p3, d)
      v = p3 - p2
      v.length = d
      trans=Geom::Transformation.translation(v)
      return p1.transform(trans)
    end   

    #------create a new set of points from the original 3 point pick
    @pt0=Geom::Point3d.new(@pts[0][0],@pts[0][1],@pts[0][2])
    @pt1=Geom::Point3d.new(@pts[1][0],@pts[1][1],@pts[1][2])
    @pt2=Geom::Point3d.new(@pts[2][0],@pts[2][1],@pts[2][2])
    @pt3=Geom::Point3d.new(@pts[3][0],@pts[3][1],@pts[3][2])

    #------find the points for the head btw. @pt0 & #pt3
    @pt0j=translate(@pt0, @pt0, @pt3, $dpFT29)
    #------find the points for the head btw. @pt1 & #pt2
    @pt1j=translate(@pt1, @pt1, @pt2, $dpFT29)
    #------find the points for the sill btw. @pt0 & #pt3
    @pt2j=translate(@pt2, @pt2, @pt1, $dpFT29)
    #------find the points for the sill btw. @pt1 & #pt2
    @pt3j=translate(@pt3, @pt3, @pt0, $dpFT29)
    #------find the points for the left jamb btw. @pt0j & #pt1j
    @pt0jj=translate(@pt0j, @pt0j, @pt1j, $dpFT29)
    #------find the points for the left jamb btw. @pt3j & #pt2j
    @pt3jj=translate(@pt3j, @pt3j, @pt2j, $dpFT29)
    #------find the points for the right jamb btw. @pt0 & #pt3
    @pt1jj=translate(@pt1j, @pt1j, @pt0j, $dpFT29)
    #------find the points for the right jamb btw. @pt1 & #pt2
    @pt2jj=translate(@pt2j, @pt2j, @pt3j, $dpFT29)

    #------width of opening minus framewidth ($dpFT29)
    @vLength=@pt0jj.distance @pt3jj
    #------height of opening minus framewidth ($dpFT29)
    @hLength=@pt1jj.distance @pt0jj 
    #------width of pane
    @vPane=(@vLength - ($jwEN56 -1) * $voVP26)/$jwEN56
    #------height of pane
    @hPane=(@hLength - ($wbTH20 -1) * $voVP26)/$wbTH20
    
    #------points required to create the 1st pane
    @pt4jj=translate(@pt3jj, @pt3jj, @pt2jj, @hPane)
    @pt6jj=translate(@pt3jj, @pt3jj, @pt0jj, @vPane)
    @pt7jj=translate(@pt0jj, @pt0jj, @pt1jj, @hPane)
    @pt5jj=translate(@pt4jj, @pt4jj, @pt7jj, @vPane)
    
    @hVec = @pt4jj - @pt3jj
    @vVec = @pt6jj - @pt3jj  
#-------------------------------------------------------------------------    
if( @gtype == "Mullions" )   
#------------------------------------------------------------------------- 
    group=entities.add_group
    entities=group.entities 
    #base=entities.add_face(@pt0,@pt1,@pt2,@pt3)
    base=entities.add_face(@pt0jj,@pt1jj,@pt2jj,@pt3jj)
    base.pushpull $coKX56
    
    1.upto($jwEN56) do |i|
      x = (@vPane*i)+(i*$voVP26)  
      1.upto($wbTH20) do |j|
	    y = (@hPane*j)+(j*$voVP26) 
        
	o = @pt3jj.offset(@hVec, y - $voVP26).offset(@vVec, x - $voVP26)
    pt1 = o.offset(@hVec, -@hPane)
	pt2 = o.offset(@vVec, -@vPane)
	pt3 = pt1.offset(@vVec, -@vPane)
    face = entities.add_face o, pt1, pt3, pt2
    face.pushpull -$coKX56
        
	#entities.add_text( "(#{x}, #{y})", [x, y] ) #view co-ord's on screen
      end
    end
end #if        
#-------------------------------------------------------------------------    
if( @gtype == "套框平板" )   
#-------------------------------------------------------------------------
    group=entities.add_group
    entities=group.entities 
      
    1.upto($jwEN56) do |i|
      x = (@vPane*i)+(i*$voVP26)  
      1.upto($wbTH20) do |j|
	    y = (@hPane*j)+(j*$voVP26) 
        
	o = @pt3jj.offset(@hVec, y - $voVP26).offset(@vVec, x - $voVP26)
    pt1 = o.offset(@hVec, -@hPane)
	pt2 = o.offset(@vVec, -@vPane)
	pt3 = pt1.offset(@vVec, -@vPane)
    face = entities.add_face o, pt1, pt3, pt2
    face.pushpull -$coKX56
        
	#entities.add_text( "(#{x}, #{y})", [x, y] ) #view co-ord's on screen
      end
    end
end #if    
#------------------------------------------------------------------------- 
if( @gtype == "竖窗与套框板" )   
#------------------------------------------------------------------------- 
    group=entities.add_group
    entities=group.entities 
    base=entities.add_face(@pt0,@pt1,@pt2,@pt3)
    #base=entities.add_face(@pt0jj,@pt1jj,@pt2jj,@pt3jj)
    base.pushpull $smBO73
    
    1.upto($jwEN56) do |i|
      x = (@vPane*i)+(i*$voVP26)  
      1.upto($wbTH20) do |j|
	    y = (@hPane*j)+(j*$voVP26) 
        
	o = @pt3jj.offset(@hVec, y - $voVP26).offset(@vVec, x - $voVP26)
    pt1 = o.offset(@hVec, -@hPane)
	pt2 = o.offset(@vVec, -@vPane)
	pt3 = pt1.offset(@vVec, -@vPane)
    face = entities.add_face o, pt1, pt3, pt2
    face.pushpull -$coKX56
        
	#entities.add_text( "(#{x}, #{y})", [x, y] ) #view co-ord's on screen
      end
    end
end #if   

    #------Reset the layer back to default layer [0]
	layers = model.layers
    activelayer = model.active_layer=layers[0]
	layer=model.active_layer
 
    model.commit_operation
end

end # class DoorToolMullion
