=begin
# Author:         tomot
# Name:           DoorToolTrim
# Description:    Create a simple DoorTrim 
# Usage:          click 3 points, into an EXISTING OPENING, creates a DoorTrim/Moulding around the 3 sides of the opening
# Date:           2008 
# Type:           Tool
# Revised:        
#------------------------------------------------------------------------------------------------
=end
require 'sketchup.rb'
require 'doortools/3PointTool'

class DoorToolTrim < ThreePointTool

def initialize

    super # calls the initialize method in the ThreePointTool class   
    # sets the default Door settings
    $axHB38 = 6.0. inch if not $axHB38          # door frame/wall thickness
    $glCS20 = 3.inch if not $glCS20             #door moulding trim width
    $igLK37 = 1.inch if not $igLK37             #door moulding trim thickness
    @tside = "Single" if not @tside             #single or both sides 
    
    # Set the layer names
    $drDR00= "门" if not $drDR00        #layer for door
             
    # Dialog box
    tside = ["单独", "同时"]
    enums = ["", "", tside.join("|")]
    prompts = ["默认图层", "墙厚", "门造型面", "门造型宽度", "门造型厚度"]
    values = [$drDR00, $axHB38, @tside, $glCS20, $igLK37]
    title = "门造型参数"
    
    results = inputbox(prompts, values, enums, title)
    
    return nil if not results
    $drDR00, $axHB38, @tside, $glCS20, $igLK37 = results
    
    $igLK37 = 0.25.inch if $igLK37 <= 0.inch
            
end #end initialize
#----------------------
def create_geometry
#----------------------

    model=Sketchup.active_model
    #------Set and activate the Doors layer
    layers = model.layers
    layers.add ($drDR00)
    activelayer = model.active_layer=layers[$drDR00]
    layer=model.active_layer
    
    model.start_operation "创建门"
    entities=model.active_entities
    
    # creates a new point from p1 in the direction of p2-p3 with length d
    # params are Point3d, 2 vertices, a length, returns a Point3d
    def translate(p1, p2, p3, d)
        v = p3 - p2
        v.length = d
        trans=Geom::Transformation.translation(v)
        return p1.transform(trans)
    end 
    
    #------create a new set of points from the original 3 point pick
    @pt0=Geom::Point3d.new(@pts[0][0], @pts[0][1], @pts[0][2])
    @pt1=Geom::Point3d.new(@pts[1][0], @pts[1][1], @pts[1][2])
    @pt2=Geom::Point3d.new(@pts[2][0], @pts[2][1], @pts[2][2])
    @pt3=Geom::Point3d.new(@pts[3][0], @pts[3][1], @pts[3][2])
         
    @pt0a=translate(@pt0, @pt0, @pt1, -$glCS20) 
    @pt1a=translate(@pt1, @pt1, @pt0, -$glCS20) 
    @pt2t=translate(@pt2, @pt2, @pt3, -$glCS20) 
    @pt3t=translate(@pt3, @pt3, @pt2, -$glCS20) 
    @pt0t=translate(@pt0a, @pt0a, @pt3t, -$glCS20) 
    @pt1t=translate(@pt1a, @pt1a, @pt2t, -$glCS20) 
    
if( @tside == "单独" )     
    #------Do Exterior Trim
    group=entities.add_group
    entities=group.entities
    base=entities.add_face(@pt0, @pt1, @pt2, @pt2t, @pt1t, @pt0t, @pt3t, @pt3)
    base.pushpull $igLK37
end #if    

if( @tside == "同时" ) 
    #------Do Exterior Trim
    group=entities.add_group
    entities=group.entities
    base=entities.add_face(@pt0, @pt1, @pt2, @pt2t, @pt1t, @pt0t, @pt3t, @pt3)
    base.pushpull $igLK37
    #------Make another
    group=entities.add_group
    entities=group.entities
    base=entities.add_face(@pt0, @pt1, @pt2, @pt2t, @pt1t, @pt0t, @pt3t, @pt3)
    #------Move to opposite wall location
    normal = base.normal
    normal.length =  $axHB38
    tr = Geom::Transformation.translation(normal.reverse)
    group.transform! tr
    
    base.pushpull -$igLK37
end #if 
        
    #------Reset the layer back to default layer [0]
	layers = model.layers
    activelayer = model.active_layer=layers[0]
	layer=model.active_layer
    
     model.commit_operation
end
  
end # class DoorToolTrim
